@@ -3,7 +3,7 @@ from magickpy.util import wrap_ptr_class
 from magickpy.enums import *
 from magickpy.types import (_ExceptionInfo, ExceptionInfo,
     TimerInfo, ProfileInfo, ImageMagickException,
-    GeometryInfo,
+    GeometryInfo, QuantizeInfo,
     PixelPacket, Color, RectangleInfo, ChromaticityInfo, ErrorInfo)
 import ctypes
 
@@ -213,22 +213,22 @@ class PixelWrapper(object):
 
     def __setitem__(self, coord, value):
         x, y = coord
-        if x > self.w or y > self.h or x < 0 or y < 0:
-            return ValueError("Wrong coordinates %d, %d" % (self.x, self.y))
+        if x >= self.w or y >= self.h or x < 0 or y < 0:
+            raise ValueError("Wrong coordinates %d, %d" % (x, y))
         if not isinstance(value, PixelPacket):
             value = PixelPacket(*value)
         self.px[0][y*self.w+x] = value
 
     def __getitem__(self, coord):
         x, y = coord
-        if x > self.w or y > self.h or x < 0 or y < 0:
-            return ValueError("Wrong coordinates %d, %d" % (self.x, self.y))
+        if x >= self.w or y >= self.h or x < 0 or y < 0:
+            raise ValueError("Wrong coordinates %d, %d" % (x, y))
         return self.px[0][y*self.w+x]
 
 class Image(_PImage):
     @classmethod
     def read(C, file):
-        if isinstance(file, basestring):
+        if isinstance(file, str):
             inf = ImageInfo()
             inf.filename = file
             exinfo = ExceptionInfo()
@@ -241,7 +241,7 @@ class Image(_PImage):
 
     @classmethod
     def tile(C, file, width, height):
-        if isinstance(file, basestring):
+        if isinstance(file, str):
             inf = ImageInfo()
             file = 'tile:' + file
             inf.size = "%dx%d" % (width, height)
@@ -260,7 +260,7 @@ class Image(_PImage):
 
     @classmethod
     def ping(C, file):
-        if isinstance(file, basestring):
+        if isinstance(file, str):
             inf = ImageInfo()
             inf.filename = file
             exinfo = ExceptionInfo()
@@ -283,7 +283,7 @@ class Image(_PImage):
         return im
 
     def write(self, file):
-        if isinstance(file, basestring):
+        if isinstance(file, str):
             inf = ImageInfo()
             self.filename = file
             if not lib.WriteImage(inf, self):
@@ -298,7 +298,7 @@ class Image(_PImage):
             if not res:
                 raise ImageMagickException(self.exception)
 
-    def __nonzero__(self):
+    def __bool__(self):
         return True
 
     @property
@@ -311,7 +311,7 @@ class Image(_PImage):
 
     def draw(self, string):
         inf = DrawInfo()
-        buf = ctypes.c_buffer(string)
+        buf = ctypes.c_buffer(string.encode('utf-8'))
         inf.primitive = ctypes.cast(buf, ctypes.c_char_p)
         try:
             if not lib.DrawImage(self.value, inf):
@@ -326,7 +326,7 @@ class Image(_PImage):
         return self._makeCrop(geom)
 
     def makeColorize(self, color, opacity_r, opacity_g=None, opacity_b=None):
-        if isinstance(opacity_r, basestring):
+        if isinstance(opacity_r, str):
             opacity = opacity_r
         else:
             if opacity_g is None:
@@ -334,7 +334,7 @@ class Image(_PImage):
             if opacity_b is None:
                 opacity_b = opacity_r
             opacity = "%u/%u/%u" % (opacity_r, opacity_g, opacity_b)
-        return self._makeColorize(opacity, color)
+        return self._makeColorize(opacity.encode('ascii'), color)
 
     def copyPixels(self, source, sx, sy, w, h, tx, ty):
         exc = ExceptionInfo()
@@ -348,7 +348,7 @@ class Image(_PImage):
         artype1 = PixelPacket*(w*h)
         dest = ctypes.cast(dest, artype)
         src = ctypes.cast(src, artype)
-        for i in xrange(w*h):
+        for i in range(w*h):
             dest[0][i] = src[0][i]
         if not lib.SyncAuthenticPixels(self, exc):
             raise ImageMagickException(exc)
@@ -371,13 +371,21 @@ class Image(_PImage):
     makeExtent = new_image_wrapper(lib.ExtentImage, ctypes.POINTER(RectangleInfo))
     makeBorder = new_image_wrapper(lib.BorderImage, ctypes.POINTER(RectangleInfo))
 
-    applyContrastStretch = apply_image_wrapper(lib.ContrastStretchImage, ctypes.c_char_p)
+    _applyContrastStretch = apply_image_wrapper(lib.ContrastStretchImage, ctypes.c_char_p)
     applyNormalize = apply_image_wrapper(lib.NormalizeImage)
     applyComposite = apply_image_wrapper(lib.CompositeImage, CompositeOp, _PImage, ctypes.c_int, ctypes.c_int)
-    applySigmoidalContrast = apply_image_wrapper(lib.SigmoidalContrastImage, ctypes.c_int, ctypes.c_char_p)
+    _applySigmoidalContrast = apply_image_wrapper(lib.SigmoidalContrastImage, ctypes.c_int, ctypes.c_char_p)
     applySeparateChannel = apply_image_wrapper(lib.SeparateImageChannel, ChannelType)
     applyNegate = apply_image_wrapper(lib.NegateImage, ctypes.c_int)
 
+    def applyQuantize(self, number_colors, dither=DitherMethod.No,
+        colorspace=ColorspaceType.RGB, measure_error=False):
+        qi = QuantizeInfo()
+        qi.dither_method = dither
+        qi.dither = dither != DitherMethod.No
+        qi.number_colors = number_colors
+        lib.QuantizeImage(qi, self)
+
     def applyDissolve(self, im, x=0, y=0, percent=None, dst_percent=None):
         g = im.geometry
         s = dst_percent is not None and "%fx%f" % (percent, dst_percent) or "%f" % percent
@@ -387,6 +395,12 @@ class Image(_PImage):
         finally:
             im.geometry = g
 
+    def applyContrastStretch(self, val):
+        self._applyContrastStretch(val.encode('ascii'))
+
+    def applySigmoidalContrast(self, a, b):
+        self._applySigmoidalContrast(a, b.encode('ascii'))
+
     def compare(self, other, metric):
         dbl = (ctypes.c_double*1)()
         exp = ExceptionInfo()
